-- This module provides a decidability predicate. A property "A" is
-- decidable is "A ∨ Not A" holds.

open import Proposition
open import Boolean

module Decidable where

-- ----------------------------------------------------------------------
-- * Definition
  
-- The type "Decidable A" is isomorphic to "A ∨ Not A". It can be used
-- as a type returned by decision procedures. Rather than just
-- deciding whether a proposition is true or false, such a procedure
-- can return a proof of A or a proof of Not A.

data Decidable (A : Set) : Set where
  yes : A -> Decidable A
  no : Not A -> Decidable A

-- ----------------------------------------------------------------------
-- * Operations

-- Extract a boolean from a decidable property.
boolean-of-decidable : ∀ {A} -> Decidable A -> Boolean
boolean-of-decidable (yes _) = true
boolean-of-decidable (no _) = false
