-- The proof of Equation 25.

import Proposition
import Equality

import Word
import Generator
import Word-Lemmas
import Word-Tactics
import Clifford-Lemmas
import CliffordT-Lemmas
import PauliRotations

open Proposition
open Equality using (auto)

open Word
open Generator
open Generator.Clifford+T
open Word-Lemmas.Monoid-Equational
open Word-Tactics.InContext
open Word-Tactics.Associative
open Clifford-Lemmas.Clifford-Rewriting
open CliffordT-Lemmas
open PauliRotations.Pauli
open PauliRotations.PauliRotRep
open PauliRotations.Commutativity
open PauliRotations.Combine
open PauliRotations.Correction

private module C = PauliRotations.Clifford

eqn25 : Clifford+T.Rel ⊢ T0⁻¹ • T1⁻¹ • W • T1 • CX1 • T1⁻¹ • ε === CX1 • T0⁻¹ • T1⁻¹ • W • ε
eqn25 = 
  equational T0⁻¹ • T1⁻¹ • W • T1 • CX1 • T1⁻¹ • ε
          by in-context 0 1 auto lemma-R-T0⁻¹
      equals R (plus , Z⊗I) • S0⁻¹ • T1⁻¹ • W • T1 • CX1 • T1⁻¹ • ε
          by in-context 2 1 auto lemma-R-T1⁻¹
      equals R (plus , Z⊗I) • S0⁻¹ • R (plus , I⊗Z) • S1⁻¹ • W • T1 • CX1 • T1⁻¹ • ε
          by in-context 5 1 auto lemma-R-T1
      equals R (plus , Z⊗I) • S0⁻¹ • R (plus , I⊗Z) • S1⁻¹ • W • R (plus , I⊗Z) • CX1 • T1⁻¹ • ε
          by in-context 7 1 auto lemma-R-T1⁻¹
      equals R (plus , Z⊗I) • S0⁻¹ • R (plus , I⊗Z) • S1⁻¹ • W • R (plus , I⊗Z) • CX1 • R (plus , I⊗Z) • S1⁻¹ • ε
          by in-context 1 2 auto (lemma-R (C.S0⁻¹) (plus , I⊗Z))
      equals R (plus , Z⊗I) • R (plus , I⊗Z) • S0⁻¹ • S1⁻¹ • W • R (plus , I⊗Z) • CX1 • R (plus , I⊗Z) • S1⁻¹ • ε
          by in-context 2 4 auto (lemma-R (C.S0⁻¹ • C.S1⁻¹ • C.W) (plus , I⊗Z))
      equals R (plus , Z⊗I) • R (plus , I⊗Z) • R (plus , I⊗Z) • S0⁻¹ • S1⁻¹ • W • CX1 • R (plus , I⊗Z) • S1⁻¹ • ε
          by in-context 3 5 auto (lemma-R (C.S0⁻¹ • C.S1⁻¹ • C.W • C.CX1) (plus , I⊗Z))
      equals R (plus , Z⊗I) • R (plus , I⊗Z) • R (plus , I⊗Z) • R (plus , Z⊗Z) • S0⁻¹ • S1⁻¹ • W • CX1 • S1⁻¹ • ε
          by in-context 0 2 auto (symm (tactic-pauli-commute (plus , I⊗Z) (plus , Z⊗I) auto))
      equals R (plus , I⊗Z) • R (plus , Z⊗I) • R (plus , I⊗Z) • R (plus , Z⊗Z) • S0⁻¹ • S1⁻¹ • W • CX1 • S1⁻¹ • ε
          by in-context 1 2 auto (symm (tactic-pauli-commute (plus , I⊗Z) (plus , Z⊗I) auto))
      equals R (plus , I⊗Z) • R (plus , I⊗Z) • R (plus , Z⊗I) • R (plus , Z⊗Z) • S0⁻¹ • S1⁻¹ • W • CX1 • S1⁻¹ • ε
          by in-context 0 2 auto (lemma-combine plus plus I⊗Z)
      equals Swap • S0 • Swap • R (plus , Z⊗I) • R (plus , Z⊗Z) • S0⁻¹ • S1⁻¹ • W • CX1 • S1⁻¹ • ε
          by clifford-tactic 0 3 10000 auto
      equals S1 • R (plus , Z⊗I) • R (plus , Z⊗Z) • S0⁻¹ • S1⁻¹ • W • CX1 • S1⁻¹ • ε
          by in-context 0 2 auto (lemma-R (C.S1) (plus , Z⊗I))
      equals R (plus , Z⊗I) • S1 • R (plus , Z⊗Z) • S0⁻¹ • S1⁻¹ • W • CX1 • S1⁻¹ • ε
          by in-context 1 2 auto (lemma-R (C.S1) (plus , Z⊗Z))
      equals R (plus , Z⊗I) • R (plus , Z⊗Z) • S1 • S0⁻¹ • S1⁻¹ • W • CX1 • S1⁻¹ • ε
          by clifford-tactic 2 6 10000 auto
      equals R (plus , Z⊗I) • R (plus , Z⊗Z) • H1 • CZ • S0 • S0 • S0 • H1 • S1 • S1 • S1 • W • ε
          by clifford-tactic 2 9 10000 auto
      equals R (plus , Z⊗I) • R (plus , Z⊗Z) • CX1 • S0⁻¹ • S1⁻¹ • W • ε
          by in-context 1 3 auto (symm (lemma-R (C.CX1 • C.S0⁻¹) (plus , I⊗Z)))
      equals R (plus , Z⊗I) • CX1 • S0⁻¹ • R (plus , I⊗Z) • S1⁻¹ • W • ε
          by in-context 0 2 auto (symm (lemma-R (C.CX1) (plus , Z⊗I)))
      equals CX1 • R (plus , Z⊗I) • S0⁻¹ • R (plus , I⊗Z) • S1⁻¹ • W • ε
          by in-context 3 2 auto (symm lemma-R-T1⁻¹)
      equals CX1 • R (plus , Z⊗I) • S0⁻¹ • T1⁻¹ • W • ε
          by in-context 1 2 auto (symm lemma-R-T0⁻¹)
      equals CX1 • T0⁻¹ • T1⁻¹ • W • ε


