-- This module provides the Maybe type.

open import Boolean
open import Equality

module Maybe where

-- The type "Maybe A" is like A, with one new element added. It can be
-- used to encode partial functions.
data Maybe (A : Set) : Set where
  Just : A -> Maybe A
  Nothing : Maybe A

-- Check whether an element of Maybe A is actually an element of A.
isJust : {A : Set} -> Maybe A -> Boolean
isJust (Just x) = true
isJust Nothing = false

-- Conversion from Maybe A to A. This requires a proof of isJust x.
fromJust : {A : Set} (x : Maybe A) -> isJust x == true -> A
fromJust {A} (Just x) hyp = x
fromJust {A} Nothing ()
